/*
 ************************************************************************
 *
 * 							The moving plane
 *
 ************************************************************************
 */

#include "image.h"
#include "ImageViews.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>



B_DEFINE_CLASS_INFO(MyPlane, BLooper);

MyPlane::MyPlane(BWindow& _top_view_win, 
				 BWindow& _cockpit_view_win,
				 ViewerPosition& _viewer_position)
	: top_view_win(_top_view_win), 
	  cockpit_view_win(_cockpit_view_win),
	  viewer_position(_viewer_position),
	  fRadius(10.0)
	  //fBoundary(_top_view_win.Bounds()),
	  //fSpeed(BPoint(10.0,5.0))
{
	/*
	 The ball object actually runs in its own thread so
	 that it is independent of any window. It sends the
	 window messages as needed.
	*/
//	BRect	b = rect;
//	b.InsetBy(10.0, 10.0);

//	fCenter = center;
//	fLength = sqrt((fSpeed.x*fSpeed.x) + (fSpeed.y*fSpeed.y));
	fSleep = 40.0;
	fPercentRemaining = 1.0;
	fEnabled = TRUE;

	Run();
	// post initial message to get things rolling
	PostMessage(BB_TICK);
}

/*--------------------------------------------------------*/

#if 0
void MyPlane::Draw(BView *view)
{
	// The plane draws itself in the given view
//	Lock();
	rgb_color	c = view->HighColor();

	view->SetPenSize(1.0);
	view->SetHighColor(150, 30, 30);
	view->FillArc(fCenter, fRadius, fRadius, 0, 360);
	view->SetHighColor(c);
//	Unlock();
}
#endif

/*--------------------------------------------------------*/

void MyPlane::Tick()
{
	/*
	 This method is repeatedly call to simulate the ball's
	 motion. The ball will wake up every fSnooze millisecs
	 and update its position.

	 So basically this is where the hit-testing and logic
	 lives for bouncing the ball around the window and
	 deciding when the ball hits the 'gap' and gets
	 teleported over to the partner application.
	*/
//	float	angle;
	BRect	updateRect;

#if 0
	if ((rand() % 3000) == 0) {
		// add a little random bahavior to the ball.
		long r = (rand() % 3) - 1.0;
		fSpeed.x += r;
		fSpeed.y -= r;
		fLength = sqrt((fSpeed.x*fSpeed.x) +
						(fSpeed.y*fSpeed.y));
	}
#endif

	/*
	 To minimize blitting we'll calculate the rect that
	 encloses the before & after locations of the ball
	 and only redraw that portion of the screen. That's
	 what 'updateRect' will do.
	*/
	updateRect = Bounds();
	// This will move the ball to its new location.
    viewer_position.do_one_turn();

//	NextPosition();

	updateRect = updateRect | Bounds();
	updateRect.InsetBy(-1, -1);

	// inform the window to redraw the window.
	{
	  BMessage	*msg = new BMessage(BB_DRAW);
	  msg->AddRect("update", updateRect);
	  top_view_win.PostMessage(msg);
	}	// the msg is "consumed" by the system. Don't use it again
	{
	  BMessage	*msg = new BMessage(BB_DRAW);
	  msg->AddRect("update", updateRect);
	  cockpit_view_win.PostMessage(msg);
	}	// the msg is "consumed" by the system. Don't use it again
#if 0

#if 0
	if (hit_hole) {
		/*
		 The 'gap' was hit. So we package up the info
		 like speed and relative location, which gives
		 our partner enough information to have the
		 ball appear in the correct place.
		*/
		float rel_loc = CalcGapPosition(angle, fGapStart,
			fGapEnd);

		BMessage *msg = new BMessage(BB_HIT_HOLE);
		msg->AddPoint("speed", fSpeed);
		msg->AddFloat("rel_loc", rel_loc);
		fWindow->PostMessage(msg);
		fEnabled = FALSE;
	}
#endif
#endif
}

/*--------------------------------------------------------*/

void MyPlane::MessageReceived(BMessage *msg)
{
	if (!fEnabled)
		return;

	switch (msg->what) {
		case BB_TICK:
			{
			//Lock();
			Tick();

			/*
			 Adjust the 'snooze' duration to keep the
			 animation flowing smoothly.
			*/
			float sleep_for = fPercentRemaining * fSleep;
			//Unlock();

			snooze(sleep_for * 1000.0);
			PostMessage(BB_TICK);
			break;
			}
	}
}

/*--------------------------------------------------------*/

BRect MyPlane::Bounds()
{
	return BRect(viewer_position.xe - fRadius, viewer_position.ye - fRadius,
				viewer_position.xe + fRadius, viewer_position.ye + fRadius);
}

#if 0

/*--------------------------------------------------------*/

void MyPlane::NextPosition(void)
{
	/*
	 Move the ball to it's next position.
	*/
	BPoint	move_by(fSpeed.x * fPercentRemaining,
					fSpeed.y * fPercentRemaining);
	BPoint	p1 = fCenter;
	BPoint	p2 = fCenter + move_by;
	BRect	b = fBoundary;

	b.InsetBy(BPoint(fRadius, fRadius));

	if (!b.Contains(p2)) {
		/*
		 The ball is now outside the boundary - so it either
		 hit a wall of it is going through the 'gap'.
		 Calculate the intersection point and the ball's
		 new speed (i.e. direction after the bounce).
		*/

//		PRINT(("\nCalcIntersection\n"));
//		PRINT_OBJECT(b); PRINT_OBJECT(p1); PRINT_OBJECT(p2); PRINT_OBJECT(fSpeed);

		BPoint new_speed = fSpeed;
		//BPoint hit = ::CalcIntersection(p1, p2, b, &new_speed);

		//p2 = hit;

			/*
			 Didn't hit the gap. The ball's new position
			 will be at the point of impact with the wall.
			 We do this so that the animation will show
			 the ball actually hitting the wall. Because
			 of this will need to adjust the 'snooze'
			 period until the next animation to keep to
			 speed of the ball relatively constant.
			*/
			fSpeed = new_speed;
			//float partial = segment_length(p1, hit);
			//fPercentRemaining -= (partial / fLength);

	fCenter = p2;	// finally update the ball's position
}
}
#endif